@@ -1,5 +1,4 @@
 package Pegex::Grammar;
-$Pegex::Grammar::VERSION = '0.32';
 use Pegex::Base;
 
 # Grammar can be in text or tree form. Tree will be compiled from text.
@@ -19,13 +18,20 @@ sub make_text {
     my ($self) = @_;
     my $filename = $self->file
         or return '';
-    open TEXT, $filename
+    open TEXT, "<", $filename
         or die "Can't open '$filename' for input\n:$!";
     return do {local $/; <TEXT>}
 }
 
 sub make_tree {
     my ($self) = @_;
+    # Turn off ENV debugging for grammar compile step:
+    local (
+        $ENV{PERL_PEGEX_DEBUG},
+        $ENV{PERL_PEGEX_RECURSION_LIMIT},
+        $ENV{PERL_PEGEX_RECURSION_WARN_LIMIT},
+        $ENV{PERL_PEGEX_ITERATION_LIMIT},
+    );
     my $text = $self->text
         or die "Can't create a '" . ref($self) .
             "' grammar. No tree or text or file.";
@@ -67,7 +73,7 @@ sub import {
 sub compile_into_module {
     my ($package) = @_;
     my $grammar_file = $package->file;
-    open GRAMMAR, $grammar_file
+    open GRAMMAR, "<", $grammar_file
         or die "Can't open $grammar_file for input";
     my $grammar_text = do {local $/; <GRAMMAR>};
     close GRAMMAR;
@@ -88,14 +94,23 @@ sub compile_into_module {
         require Pegex::Compiler;
         $perl = Pegex::Compiler->new->compile($grammar_text, @rules)->to_perl;
     }
-    open IN, $file or die $!;
+    open IN, "<", $file or die $!;
     my $module_text = do {local $/; <IN>};
+    require Pegex;
+    my $msg = "   # Generated/Inlined by Pegex::Grammar ($Pegex::VERSION)";
     close IN;
     $perl =~ s/^/  /gm;
-    $module_text =~ s/^(sub\s+make_tree\s*\{).*?(^\})/$1\n$perl$2/ms;
+    $module_text =~ s/^(sub\s+make_tree\s*\{).*?(^\})/$1$msg\n$perl$2/ms;
+    $module_text =~ s/^(sub\s+tree\s*\{).*?(^\})/$1$msg\n$perl$2/ms;
+    chomp $grammar_text;
+    $grammar_text = "<<'...';\n$grammar_text\n...\n";
+    $module_text =~ s/^(sub\s+text\s*\{).*?(^\})/$1$msg\n$grammar_text$2/ms;
+    $grammar_text =~ s/^/# /gm;
+    $module_text =~ s/^(# sub\s+text\s*\{).*?(^# \})/$1$msg\n$grammar_text$2/ms;
     open OUT, '>', $file or die $!;
     print OUT $module_text;
     close OUT;
+    print "Compiled '$grammar_file' into '$file'.\n";
 }
 
 1;
